pub type ProvidedValue<'tcx> = ConstValue<'tcx>;
This type alias specifies the type returned from query providers and the type used for decoding. For regular queries this is the declared returned type V, but arena_cache will use <V as Deref>::Target instead.

Aliased Type

enum ProvidedValue<'tcx> {
    Scalar(Scalar<AllocId>),
    ZeroSized,
    Slice {
        data: ConstAllocation<'tcx>,
        meta: u64,
    },
    Indirect {
        alloc_id: AllocId,
        offset: Size,
    },
}

Variants



Scalar(Scalar<AllocId>)

Used for types with layout::abi::Scalar ABI.

Not using the enum Value to encode that this must not be Uninit.



ZeroSized

Only for ZSTs.



Slice

Fields

data: ConstAllocation<'tcx>

The allocation storing the slice contents. This always points to the beginning of the allocation.

meta: u64

The metadata field of the reference. This is a “target usize”, so we use u64 as in the interpreter.

Used for references to unsized types with slice tail.

This is worth an optimized representation since Rust has literals of type &str and &[u8]. Not having to indirect those through an AllocId (or two, if we used Indirect) has shown measurable performance improvements on stress tests. We then reuse this optimization for slice-tail types more generally during valtree-to-constval conversion.



Indirect

Fields

alloc_id: AllocId

The backing memory of the value. May contain more memory than needed for just the value if this points into some other larger ConstValue.

We use an AllocId here instead of a ConstAllocation<'tcx> to make sure that when a raw constant (which is basically just an AllocId) is turned into a ConstValue and back, we can preserve the original AllocId.

offset: Size

Offset into alloc

A value not representable by the other variants; needs to be stored in-memory.

Must not be used for scalars or ZST, but having &str or other slices in this variant is fine.

Implementations

source

impl<'tcx> ConstValue<'tcx>

source

pub fn try_to_scalar(&self) -> Option<Scalar<AllocId>>

source

pub fn try_to_scalar_int(&self) -> Option<ScalarInt>

source

pub fn try_to_bits(&self, size: Size) -> Option<u128>

source

pub fn try_to_bool(&self) -> Option<bool>

source

pub fn try_to_target_usize(&self, tcx: TyCtxt<'tcx>) -> Option<u64>

source

pub fn try_to_bits_for_ty( &self, tcx: TyCtxt<'tcx>, param_env: ParamEnv<'tcx>, ty: Ty<'tcx> ) -> Option<u128>

source

pub fn from_bool(b: bool) -> Self

source

pub fn from_u64(i: u64) -> Self

source

pub fn from_u128(i: u128) -> Self

source

pub fn from_target_usize(i: u64, cx: &impl HasDataLayout) -> Self

source

pub fn try_get_slice_bytes_for_diagnostics( &self, tcx: TyCtxt<'tcx> ) -> Option<&'tcx [u8]>

Must only be called on constants of type &str or &[u8]!